#!/bin/bash

# This work was created by participants in the DataONE project, and is
# jointly copyrighted by participating institutions in DataONE. For
# more information on DataONE, see our web site at http://dataone.org.
#
#   Copyright 2014
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Dumps system metadata related tables from the Coordinating Node that the script is run on into a
# .sql dump file.
# 2) Modifies the SQL dump to append a prefix to entity names based on the hostname of the
# originating CN (e.g. ucsb_)
# 3) Transfers the SQL dump in bulk via rsync to the target machine where the dump will be restored
# to create the new per-CN tables where the tidy merging will happen.
# Note that (4) is a manual step to run the SQL dump script on the target machine.
# Also, this script requires that the postgres user on each machine the script runs on has an
# authorized_keys file in ~/.ssh that includes the public SSH key of the postgres user on the
# target machine, and the target machine postgres user has the private SSH key installed correctly.

D1_ENV=development # change to production for production run 
if [[ "$D1_ENV" == "production" ]]; then
    HOSTID=$(hostname -s | cut -d"-" -f2);
else
    HOSTID=$(hostname -s | cut -d"-" -f3);
fi
TARGET_CN=cn-dev-ucsb-1.test.dataone.org
DUMP_DIR=".";
mkdir -p ${DUMP_DIR};
DUMP_FILE="${DUMP_DIR}/dump-$(hostname -s)-$(date +%Y%m%d%H%M).sql";
DUMP_CMD="pg_dump \
-h localhost \
-t identifier \
-t smreplicationpolicy \
-t smreplicationstatus \
-t systemmetadata \
-t xml_access \
-f ${DUMP_FILE} \
-U metacat metacat";
echo "Running: " "${DUMP_CMD}";
${DUMP_CMD};

echo "Altering SQL DDL statements ...";
echo "";
sed -i.bak -e "s/CREATE TABLE identifier/DROP TABLE IF EXISTS ${HOSTID}_identifier; CREATE TABLE ${HOSTID}_identifier/" ${DUMP_FILE};
sed -i.bak -e "s/ALTER TABLE identifier/ALTER TABLE ${HOSTID}_identifier/" ${DUMP_FILE};
sed -i.bak -e "s/ALTER TABLE public\.identifier/ALTER TABLE public\.${HOSTID}_identifier/" ${DUMP_FILE};
sed -i.bak -e "s/ALTER TABLE ONLY identifier/ALTER TABLE ONLY ${HOSTID}_identifier/" ${DUMP_FILE};
sed -i.bak -e "s/COPY identifier/COPY ${HOSTID}_identifier/" ${DUMP_FILE};
sed -i.bak -e "s/ADD CONSTRAINT identifier/ADD CONSTRAINT ${HOSTID}_identifier/" ${DUMP_FILE};
sed -i.bak -e "s/CREATE INDEX identifier/CREATE INDEX ${HOSTID}_identifier/" ${DUMP_FILE};
sed -i.bak -e "s/ON identifier/ON ${HOSTID}_identifier/" ${DUMP_FILE};
sed -i.bak -e "s/REFERENCES identifier/REFERENCES ${HOSTID}_identifier/" ${DUMP_FILE};

sed -i.bak -e "s/CREATE TABLE systemmetadata/DROP TABLE IF EXISTS ${HOSTID}_systemmetadata; CREATE TABLE ${HOSTID}_systemmetadata/" ${DUMP_FILE};
sed -i.bak -e "s/ALTER TABLE systemmetadata/ALTER TABLE ${HOSTID}_systemmetadata/" ${DUMP_FILE};
sed -i.bak -e "s/ALTER TABLE public\.systemmetadata/ALTER TABLE public\.${HOSTID}_systemmetadata/" ${DUMP_FILE};
sed -i.bak -e "s/ALTER TABLE ONLY systemmetadata/ALTER TABLE ONLY ${HOSTID}_systemmetadata/" ${DUMP_FILE};
sed -i.bak -e "s/COPY systemmetadata/COPY ${HOSTID}_systemmetadata/" ${DUMP_FILE};
sed -i.bak -e "s/ADD CONSTRAINT systemmetadata/ADD CONSTRAINT ${HOSTID}_systemmetadata/" ${DUMP_FILE};
sed -i.bak -e "s/CREATE INDEX systemmetadata/CREATE INDEX ${HOSTID}_systemmetadata/" ${DUMP_FILE};
sed -i.bak -e "s/ON systemmetadata/ON ${HOSTID}_systemmetadata/" ${DUMP_FILE};
sed -i.bak -e "s/REFERENCES systemmetadata/REFERENCES ${HOSTID}_systemmetadata/" ${DUMP_FILE};

sed -i.bak -e "s/CREATE TABLE smreplicationpolicy/DROP TABLE IF EXISTS ${HOSTID}_smreplicationpolicy; CREATE TABLE ${HOSTID}_smreplicationpolicy/" ${DUMP_FILE};
sed -i.bak -e "s/ALTER TABLE smreplicationpolicy/ALTER TABLE ${HOSTID}_smreplicationpolicy/" ${DUMP_FILE};
sed -i.bak -e "s/ALTER TABLE public\.smreplicationpolicy/ALTER TABLE public\.${HOSTID}_smreplicationpolicy/" ${DUMP_FILE};
sed -i.bak -e "s/ALTER TABLE ONLY smreplicationpolicy/ALTER TABLE ONLY ${HOSTID}_smreplicationpolicy/" ${DUMP_FILE};
sed -i.bak -e "s/COPY smreplicationpolicy/COPY ${HOSTID}_smreplicationpolicy/" ${DUMP_FILE};
sed -i.bak -e "s/ADD CONSTRAINT smreplicationpolicy/ADD CONSTRAINT ${HOSTID}_smreplicationpolicy/" ${DUMP_FILE};
sed -i.bak -e "s/CREATE INDEX smreplicationpolicy/CREATE INDEX ${HOSTID}_smreplicationpolicy/" ${DUMP_FILE};
sed -i.bak -e "s/ON smreplicationpolicy/ON ${HOSTID}_smreplicationpolicy/" ${DUMP_FILE};
sed -i.bak -e "s/REFERENCES smreplicationpolicy/REFERENCES ${HOSTID}_smreplicationpolicy/" ${DUMP_FILE};

sed -i.bak -e "s/CREATE TABLE smreplicationstatus/DROP TABLE IF EXISTS ${HOSTID}_smreplicationstatus; CREATE TABLE ${HOSTID}_smreplicationstatus/" ${DUMP_FILE};
sed -i.bak -e "s/ALTER TABLE smreplicationstatus/ALTER TABLE ${HOSTID}_smreplicationstatus/" ${DUMP_FILE};
sed -i.bak -e "s/ALTER TABLE public\.smreplicationstatus/ALTER TABLE public\.${HOSTID}_smreplicationstatus/" ${DUMP_FILE};
sed -i.bak -e "s/ALTER TABLE ONLY smreplicationstatus/ALTER TABLE ONLY ${HOSTID}_smreplicationstatus/" ${DUMP_FILE};
sed -i.bak -e "s/COPY smreplicationstatus/COPY ${HOSTID}_smreplicationstatus/" ${DUMP_FILE};
sed -i.bak -e "s/ADD CONSTRAINT smreplicationstatus/ADD CONSTRAINT ${HOSTID}_smreplicationstatus/" ${DUMP_FILE};
sed -i.bak -e "s/CREATE INDEX smreplicationstatus/CREATE INDEX ${HOSTID}_smreplicationstatus/" ${DUMP_FILE};
sed -i.bak -e "s/ON smreplicationstatus/ON ${HOSTID}_smreplicationstatus/" ${DUMP_FILE};
sed -i.bak -e "s/REFERENCES smreplicationstatus/REFERENCES ${HOSTID}_smreplicationstatus/" ${DUMP_FILE};

sed -i.bak -e "s/CREATE TABLE xml_access/DROP TABLE IF EXISTS ${HOSTID}_xml; CREATE TABLE ${HOSTID}_xml_access/" ${DUMP_FILE};
sed -i.bak -e "s/ALTER TABLE xml_access/ALTER TABLE ${HOSTID}_xml_access/" ${DUMP_FILE};
sed -i.bak -e "s/ALTER TABLE public\.xml_access/ALTER TABLE public\.${HOSTID}_xml_access/" ${DUMP_FILE};
sed -i.bak -e "s/ALTER TABLE ONLY xml_access/ALTER TABLE ONLY ${HOSTID}_xml_access/" ${DUMP_FILE};
sed -i.bak -e "s/COPY xml_access/COPY ${HOSTID}_xml_access/" ${DUMP_FILE};
sed -i.bak -e "s/ADD CONSTRAINT xml_access/ADD CONSTRAINT ${HOSTID}_xml_access/" ${DUMP_FILE};
sed -i.bak -e "s/CREATE INDEX xml_access/CREATE INDEX ${HOSTID}_xml_access/" ${DUMP_FILE};
sed -i.bak -e "s/ON xml_access/ON ${HOSTID}_xml_access/" ${DUMP_FILE};
sed -i.bak -e "s/REFERENCES xml_access/REFERENCES ${HOSTID}_xml_access/" ${DUMP_FILE};

echo "Transferring data ...";
echo ""
ssh ${TARGET_CN} mkdir -p tx_sysmeta
rsync --progress --partial -e ssh ${DUMP_FILE} ${TARGET_CN}:tx_sysmeta 

