package org.dataone.tidy.merge;

import static org.junit.Assert.*;

import java.util.Map;

import org.dataone.cn.dao.exceptions.DataAccessException;
import org.dataone.configuration.Settings;
import org.dataone.service.types.v1.Identifier;
import org.dataone.service.types.v1.NodeReference;
import org.dataone.service.types.v1.SystemMetadata;
import org.dataone.tidy.dao.ChangeRecord;
import org.dataone.tidy.dao.ChangeRecordRepository;
import org.dataone.tidy.dao.MergeResult;
import org.dataone.tidy.dao.MergeStatus;
import org.dataone.tidy.merge.strategy.FileAuthMNSysmetaProxy;
import org.junit.Before;
import org.junit.Test;

public class MergeJobTest {

	
    private static final String ORC_NODE_ID = Settings.getConfiguration().getString("tidy.dao.orc.nodeid");
    private static final String UCSB_NODE_ID = Settings.getConfiguration().getString("tidy.dao.ucsb.nodeid");
    private static final String UNM_NODE_ID = Settings.getConfiguration().getString("tidy.dao.unm.nodeid");

    
	private static MergeJob mj;
	private static Identifier pid;
	
	@Before
	public void setUp() throws Exception {
		if (mj == null) {
			pid = new Identifier();
			pid.setValue("foo");
			mj = new MergeJob(pid);

		}
		
	}

	@Test
	public final void testCall() throws Exception {
		
		MockSystemMetadataTidyDaoImpl sysmetaRepo = new MockSystemMetadataTidyDaoImpl();
		mj.setSystemMetadataTidyDao(sysmetaRepo);
		
		mj.setMnReadClient(new FileAuthMNSysmetaProxy("replicationAllowed/sysmeta_authMN.xml"));
		
		MockMergeResultRepository mergeResultRepo = new MockMergeResultRepository();
		mj.setMergeResultRepository(mergeResultRepo);
		
		MockChangeRecordRepository crRepo = new MockChangeRecordRepository();
		mj.setChangeRecordRepository(crRepo);
		
		mj.call();
		
		assertTrue("changeRecordRepo should have records saved", 
				crRepo.getChangeRecordList().size() > 0);
		
		assertTrue("resulting systemMetadata should have been saved to Sysmeta Repo", 
				sysmetaRepo.getSavedResultingSysmeta() != null);
		
		MergeResult result = mergeResultRepo.getSavedMergeResult();
		System.out.println(result);
		System.out.println(result.getMnSystemMetadataStr());
		
		assertNotNull("the mergeResult should have been saved to the MergeResult Repo", result);
		assertNotNull("the authMN sysmeta should have been save in the mergeResult", result.getMnSystemMetadataStr());
		
	}
	
	@Test
	public final void testCall_AccessPolicyDifferences() throws Exception {
		
		ResourceSystemMetadataTidyDaoImpl sysmetaRepo = 
				new ResourceSystemMetadataTidyDaoImpl("strategy/df35d.21.2/cn-orc.xml",
						"strategy/df35d.21.2/cn-unm.xml",null);
		mj.setSystemMetadataTidyDao(sysmetaRepo);
		
		mj.setMnReadClient(new FileAuthMNSysmetaProxy("df35d.21.2/authmn.xml"));
		
		MockMergeResultRepository mergeResultRepo = new MockMergeResultRepository();
		mj.setMergeResultRepository(mergeResultRepo);
		
		MockChangeRecordRepository crRepo = new MockChangeRecordRepository();
		mj.setChangeRecordRepository(crRepo);
		
		mj.call();
		
		assertTrue("changeRecordRepo should an accessPolicy differences saved", 
				crRepo.getChangeRecordList().size() > 0 && 
				crRepo.getChangeRecordList().get(1).toString().contains("MaxPermissionMap/public"));
		
		assertTrue("resulting systemMetadata should have been saved to Sysmeta Repo", 
				sysmetaRepo.getSavedResultingSysmeta() != null);
		
		MergeResult result = mergeResultRepo.getSavedMergeResult();
		System.out.println(result);
		System.out.println(result.getMnSystemMetadataStr());
		
		assertNotNull("the mergeResult should have been saved to the MergeResult Repo", result);
		assertNotNull("the authMN sysmeta should have been save in the mergeResult", result.getMnSystemMetadataStr());
		
	}
	
	
	@Test
	public final void testCall_allTheSame() throws Exception {
		
		ResourceSystemMetadataTidyDaoImpl sysmetaRepo = 
				new ResourceSystemMetadataTidyDaoImpl("strategy/allTheSame/cn-orc.xml",
						"strategy/allTheSame/cn-unm.xml","strategy/allTheSame/cn-ucsb.xml");
		mj.setSystemMetadataTidyDao(sysmetaRepo);
		
		mj.setMnReadClient(new FileAuthMNSysmetaProxy("allTheSame/authmn.xml"));
		
		MockMergeResultRepository mergeResultRepo = new MockMergeResultRepository();
		mj.setMergeResultRepository(mergeResultRepo);
		
		MockChangeRecordRepository crRepo = new MockChangeRecordRepository();
		mj.setChangeRecordRepository(crRepo);
		
		mj.call();
		
		assertTrue("changeRecordRepo should not have any differences saved", 
				crRepo.getChangeRecordList().size() == 0);
		
		assertTrue("resulting systemMetadata should have been saved to Sysmeta Repo", 
				sysmetaRepo.getSavedResultingSysmeta() != null);
		
		MergeResult result = mergeResultRepo.getSavedMergeResult();
		System.out.println(result);
		System.out.println(result.getMnSystemMetadataStr());
		
		assertNotNull("the mergeResult should have been saved to the MergeResult Repo", result);
		assertNotNull("the authMN sysmeta should have been save in the mergeResult", result.getMnSystemMetadataStr());
		
	}


	
	@Test
	public final void testCall_CaughtException() throws Exception {
		
		MockFailSaveSystemMetadataTidyDaoImpl sysmetaRepo = new MockFailSaveSystemMetadataTidyDaoImpl();
		mj.setSystemMetadataTidyDao(sysmetaRepo);
		
		mj.setMnReadClient(new FileAuthMNSysmetaProxy("replicationAllowed/sysmeta_authMN.xml"));
		
		MockMergeResultRepository mergeResultRepo = new MockMergeResultRepository();
		mj.setMergeResultRepository(mergeResultRepo);
		
		MockChangeRecordRepository crRepo = new MockChangeRecordRepository();
		mj.setChangeRecordRepository(crRepo);
		
		
		mj.call();
		
		
		assertTrue("changeRecordRepo should have records saved", 
				crRepo.getChangeRecordList().size() > 0);
		
		assertTrue("resulting systemMetadata should NOT have been saved to Sysmeta Repo", 
				sysmetaRepo.getSavedResultingSysmeta() == null);
		
		MergeResult result = mergeResultRepo.getSavedMergeResult();
		
		
		assertEquals("should report a FAILED job", result.getMergeStatus(), MergeStatus.FAILURE);
		
		System.out.println(result);
		System.out.println(result.getMnSystemMetadataStr());
		
		assertNotNull("the mergeResult should have been saved to the MergeResult Repo", result);
		assertNotNull("the authMN sysmeta should have been save in the mergeResult", result.getMnSystemMetadataStr());
		
	}

// this turned out to trace back to a problem with the live sysmetaDAO, so this test
// doesn't make a good unit test for this part of the project.
//	@Test
	public final void testCall_ExcessiveChanges_Bug() throws Exception {


		ResourceSystemMetadataTidyDaoImpl sysmetaRepo = 
				new ResourceSystemMetadataTidyDaoImpl(
						"strategy/EVAWF1Metadata20/EVAWF1Metadata20-ucsb.xml",
						"strategy/EVAWF1Metadata20/EVAWF1Metadata20-unm.xml",
						null
						);
		mj.setSystemMetadataTidyDao(sysmetaRepo);

		mj.setMnReadClient(new FileAuthMNSysmetaProxy("EVAWF1Metadata20/EVAWF1Metadata20-MNdemo6.xml"));

		MockMergeResultRepository mergeResultRepo = new MockMergeResultRepository();
		mj.setMergeResultRepository(mergeResultRepo);

		MockChangeRecordRepository crRepo = new MockChangeRecordRepository();
		mj.setChangeRecordRepository(crRepo);

		mj.call();

		assertTrue("changeRecordRepo shouldhave records saved", 
				crRepo.getChangeRecordList() != null && crRepo.getChangeRecordList().size() > 0);

		assertTrue("resulting systemMetadata should have been saved to Sysmeta Repo", 
				sysmetaRepo.getSavedResultingSysmeta() != null);

		MergeResult result = mergeResultRepo.getSavedMergeResult();
		System.out.println(result);
		System.out.println(result.getMnSystemMetadataStr());

		assertNotNull("the mergeResult should have been saved to the MergeResult Repo", result);
		assertNotNull("the authMN sysmeta should have been save in the mergeResult", result.getMnSystemMetadataStr());

	}

	
	
	/**
	 * tests call() behavior when a MergeResult cannot be persisted, especially
	 * that an exception is thrown.  Note that run outputs (mergedSystemMetadata)
	 * can be persisted.
	 * @throws Exception
	 */
	@Test
	public final void testCall_PersistMergeResultException() throws Exception {
		
		MockSystemMetadataTidyDaoImpl sysmetaRepo = new MockSystemMetadataTidyDaoImpl();
		mj.setSystemMetadataTidyDao(sysmetaRepo);
		
		mj.setMnReadClient(new FileAuthMNSysmetaProxy("replicationAllowed/sysmeta_authMN.xml"));
		
		MockChangeRecordRepository crRepo = new MockChangeRecordRepository();
		mj.setChangeRecordRepository(crRepo);
		
		MockFailedMergeResultRepository mergeResultRepo = new MockFailedMergeResultRepository();
		mj.setMergeResultRepository(mergeResultRepo);
		
		try {
			mj.call();
//			fail("call() should have thrown a (runtime) exception");
		}
		catch (Exception e) {
			if (!(e instanceof RuntimeException)) {
				fail("expecting a runtime exception");
			}
		}
		
		System.out.println(crRepo);
		System.out.println(crRepo.getChangeRecordList());
		assertTrue("changeRecordRepo should have records saved", 
				crRepo.getChangeRecordList().size() > 0);
		
		assertTrue("resulting systemMetadata should have been saved to Sysmeta Repo", 
				sysmetaRepo.getSavedResultingSysmeta() != null);
		
		MergeResult result = mergeResultRepo.getSavedMergeResult();
		
		assertNull("the mergeResult should NOT have been saved to the MergeResult Repo", result);
	}
	
	
	
	@Test
	public final void testFetchCNSysmetas() throws DataAccessException {
		mj.setSystemMetadataTidyDao(new MockSystemMetadataTidyDaoImpl());
		Map<String,SystemMetadata> map = mj.fetchCNSysmetas();
		assertTrue("should contain ORC", map.containsKey(ORC_NODE_ID));
		assertTrue("should contain UNM", map.containsKey(UNM_NODE_ID));
		assertTrue("should contain UCSB", map.containsKey(UCSB_NODE_ID));
	}

	
//	@Test
	public final void testPersistMergeReport() {
		fail("Not yet implemented");
	}

	@Test
	public final void testBuildSysmetaChangesReport() throws DataAccessException {
		mj.setSystemMetadataTidyDao(new MockSystemMetadataTidyDaoImpl());
		Map<String,SystemMetadata> map = mj.fetchCNSysmetas();
		assertTrue("should contain ORC", map.containsKey(ORC_NODE_ID));
		assertTrue("should contain UNM", map.containsKey(UNM_NODE_ID));
		assertTrue("should contain UCSB", map.containsKey(UCSB_NODE_ID));
		
		Identifier pid = new Identifier();
		pid.setValue("foo");
		MergeReport mr = new MergeReport(pid);
		mj.buildSysmetaChangesReport(map, mr);
		
		assertTrue("the ChangeRecord List should not be empty",mr.getChangeRecordList().size() > 0);
		for (ChangeRecord cr : mr.getChangeRecordList()) {
			System.out.println(cr);
		}
		
	}
	
	@Test
	public final void testBuildSysmetaChangesReport_ArchivedBizLogic() throws DataAccessException {
		mj.setSystemMetadataTidyDao(new MockSystemMetadataTidyDaoImpl());
		Map<String,SystemMetadata> map = mj.fetchCNSysmetas();
		assertTrue("should contain ORC", map.containsKey(ORC_NODE_ID));
		assertTrue("should contain UNM", map.containsKey(UNM_NODE_ID));
		assertTrue("should contain UCSB", map.containsKey(UCSB_NODE_ID));
		
		map.get(ORC_NODE_ID).setArchived(Boolean.FALSE);
		
		Identifier pid = new Identifier();
		pid.setValue("foo");
		MergeReport mr = new MergeReport(pid);
		mj.buildSysmetaChangesReport(map, mr);
		
		assertTrue("the ChangeRecord List should not be empty",mr.getChangeRecordList().size() > 0);
		for (ChangeRecord cr : mr.getChangeRecordList()) {
			if (cr.getAttribute().contains("Archived")) {
				System.out.println(cr);
				fail("Should not get a line for the Archived attribute");
			}
		}
		
	}
	
	
	@Test
	public final void testBuildSysmetaChangesReport_zeroLengthStringElements() throws DataAccessException {
		mj.setSystemMetadataTidyDao(new MockSystemMetadataTidyDaoImpl());
		Map<String,SystemMetadata> map = mj.fetchCNSysmetas();
		assertTrue("should contain ORC", map.containsKey(ORC_NODE_ID));
		assertTrue("should contain UNM", map.containsKey(UNM_NODE_ID));
		assertTrue("should contain UCSB", map.containsKey(UCSB_NODE_ID));
		
		map.get(UCSB_NODE_ID).getReplicationPolicy().clearBlockedMemberNodeList();
		map.get(UNM_NODE_ID).getReplicationPolicy().clearBlockedMemberNodeList();
		map.get(ORC_NODE_ID).getReplicationPolicy().clearBlockedMemberNodeList();
		NodeReference nodeRef = new NodeReference();
//		nodeRef.setValue("");
		map.get(ORC_NODE_ID).getReplicationPolicy().addBlockedMemberNode(nodeRef);
		
		Identifier pid = new Identifier();
		pid.setValue("foo");
		MergeReport mr = new MergeReport(pid);
		mj.buildSysmetaChangesReport(map, mr);
		
		assertTrue("the ChangeRecord List should not be empty",mr.getChangeRecordList().size() > 0);
		
		boolean fail = false;
		for (ChangeRecord cr : mr.getChangeRecordList()) {
			if (cr.getAttribute().contains("Blocked")) {
				System.out.println(cr);
				fail = true;
			}
		}
		assertFalse("Should not get a line for the BlockedMemberNodeList attribute",fail);

		
	}
	
	

	
//	@Test
	public final void testAbstractTidyJob() {
		fail("Not yet implemented");
	}

	@Test
	public final void testGetPid() {
		assertEquals("pid should match what was set in the constructor", pid, mj.getPid());
	}

	@Test
	public final void testSetPid() {
		Identifier newPid = new Identifier();
		newPid.setValue("bar");
		mj.setPid(newPid);
		assertEquals("pid should match what was set by setPid()", newPid, mj.getPid());
		mj.setPid(pid);
		assertEquals("pid should return to what was originally set by the constructor", pid, mj.getPid());
	}

// the rest of the methods from AbstractTidyJob are setters and getter, so not going to test them here.	
	
//	@Test
	public final void testGetFailure() {
		fail("Not yet implemented");
	}

//	@Test
	public final void testSetFailure() {
		fail("Not yet implemented");
	}

//	@Test
	public final void testCall1() {
		fail("Not yet implemented");
	}

//	@Test
	public final void testGetMnReadClient() {
		fail("Not yet implemented");
	}

//	@Test
	public final void testSetMnReadClient() {
		fail("Not yet implemented");
	}

//	@Test
	public final void testSetMergeResultRepository() {
		fail("Not yet implemented");
	}

//	@Test
	public final void testSetChangeRecordRepository() {
		fail("Not yet implemented");
	}

//	@Test
	public final void testSetSystemMetadataTidyDao() {
		fail("Not yet implemented");
	}

}
